# -*- coding: utf-8 -*-
#
#  __init__.py
#  simplestats
# 
#  Created by Lars Yencken on 10-04-2009.
#  Copyright 2009 Lars Yencken. All rights reserved.
#

"""
A simple statistics package.
"""

from basic import mean, stddev, basic_stats
from freq import FreqDist, ConditionalFreqDist

# vim: ts=4 sw=4 sts=4 et tw=78:

